<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns="http://www.utc.fr/ics/hdoc/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:db="http://docbook.org/ns/docbook"
    xmlns:xl="http://www.w3.org/1999/xlink"
    exclude-result-prefixes="xs db xl #default"
    version="2.0"
    >
    <xsl:output method="xml" indent="yes"/>

    <xsl:template match="db:article">
        <xsl:processing-instruction name="xml-model">
            <xsl:text>href="http://scenari.utc.fr/hdoc/schemas/xhtml/hdoc1-xhtml.rng" schematypens="http://relaxng.org/ns/structure/1.0"</xsl:text>
        </xsl:processing-instruction>
        <html xmlns="http://www.utc.fr/ics/hdoc/xhtml">
            <head>
                <xsl:apply-templates select="db:info"/>                
            </head>
            <body>
                <!-- if there is a para child of body, put it in a section child of article -->
                <xsl:if test="db:para">
                    <section>
                        <xsl:attribute name="data-hdoc-type">introduction</xsl:attribute>
                        <header><h1>Introduction</h1></header>
                            <xsl:apply-templates select="db:para"/>
                    </section>
                </xsl:if>
                <xsl:apply-templates select="db:section | db:sect1 | db:procedure"/>
            </body>
        </html>
    </xsl:template>
    <!-- DOC INFOS MATCHING !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -->
    <xsl:template match="db:info">
        <title>
            <xsl:value-of select="db:title"/>
        </title>
        <meta charset="utf-8"/>
        <meta name="generator" content="HdocConverter/Optim1.2"/>
        <xsl:variable name="subtitle" select="db:subtitle"/>
        <meta name="description" content="{$subtitle}"/>
        <xsl:apply-templates select="db:author"/>
        <xsl:apply-templates select="db:authorgroup"/>
        <xsl:apply-templates select="db:pubdate"/>
    </xsl:template>
    
    <!-- if there is an "<info>" which contains info about the author of the section in a section, put it in a div -->
    <xsl:template match="db:info[name(..) = 'section']">
        <div data-hdoc-type="author">
            <xsl:value-of select="./db:author/db:personname"/>
        </div>
    </xsl:template>
    
    <xsl:template match="db:authorgroup">
        <xsl:apply-templates select="db:author"/>
    </xsl:template>
    
    <xsl:template match="db:author">
        <xsl:variable name="name" select="./db:personname"/>
        <meta name="author" content="{$name}"/>
    </xsl:template>
        
    <xsl:template match="db:pubdate">
        <xsl:variable name="date" select="."/>
        <meta name="date" content="{$date}"/>
    </xsl:template>
    
	<!--- CONTENT MATCHING !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -->
		<!-- SECTION  -->
	    <xsl:template match="db:section">
        <section>
                <!--if the section has no nested sections, set as unit of content -->
                <xsl:if test="not(db:section)">
                    <xsl:attribute name="data-hdoc-type">unit-of-content</xsl:attribute>
                </xsl:if>
            <header>
                <h1><xsl:value-of select="db:title"/></h1>
                <xsl:apply-templates select="db:info"/>
            </header>
            <xsl:apply-templates select="*[name(.) != 'title' and name(.) != 'info' and name(.) != 'section']"/>
            <xsl:apply-templates select="db:section"/>
            <!-- if there are para or content after nested sections, put it in a conclusion section (currently doesn't work fine) -->
            <!--<section>
                <xsl:attribute name="data-hdoc-type">conclusion</xsl:attribute>
                <header>
                    <h1>Conclusion</h1>
                </header>
                <xsl:apply-templates select="*[name(.) != 'title' and name(.) != 'info' and name(.) != 'section']"/>
            </section>-->
        </section>
    </xsl:template>
    
	<!-- SECTION -->
    <xsl:template match="db:sect1 | db:sect2 | db:sect3 | db:sect4 | db:sect5 | db:sect6">
        <section>
            <!--if the section has no nested sections, set as unit of content -->
            <xsl:if test="not(db:section) and not(db:sect2) and not(db:sect3) and not(db:sect4) and not(db:sect5) and not(db:sect6)">
                <xsl:attribute name="data-hdoc-type">unit-of-content</xsl:attribute>
            </xsl:if>
            <header>
                <h1><xsl:value-of select="db:title"/></h1>
                <xsl:apply-templates select="db:info"/>
            </header>
            <xsl:apply-templates select="*[name(.) != 'title' and name(.) != 'info' and name(.) != 'sect1' and name(.) != 'sect2' and name(.) != 'sect3' and name(.) != 'sect4' and name(.) != 'sect5' and name(.) != 'sect6']"/>             
            <!-- apply template to the child of the sect in function of its level (1, 2, 3, ...) -->
            <xsl:choose>
                <xsl:when test="name(.) = 'sect1'">
                    <xsl:apply-templates select="db:sect2"/>
                </xsl:when>
                <xsl:when test="name(.) = 'sect2'">
                    <xsl:apply-templates select="db:sect3"/>
                </xsl:when>
                <xsl:when test="name(.) = 'sect3'">
                    <xsl:apply-templates select="db:sect4"/>
                </xsl:when>
                <xsl:when test="name(.) = 'sect4'">
                    <xsl:apply-templates select="db:sect5"/>
                </xsl:when>
                <xsl:when test="name(.) = 'sect5'">
                    <xsl:apply-templates select="db:sect6"/>
                </xsl:when>
            </xsl:choose>
        </section>
    </xsl:template>
	
	<!-- SECTION: if there is a footer section, put it in "footer" -->
    <xsl:template match="db:section[@label = 'footer']">
        <footer>
            <div data-hdoc-type="tags">
                <!--  para in footer section are not the same than in other situations -->
                <xsl:apply-templates select="db:para"/>
            </div>
        </footer>
    </xsl:template>
    
    <!-- PARA MATCHING !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -->
    <!-- A para is most of the time a div with a p -->
    <xsl:template match="db:para">
        <div>
            <p>
                <xsl:apply-templates select="* | text()"/>
            </p>
        </div>
    </xsl:template>
    
    <!-- if there is an itemized list in a para -->
    <xsl:template match="db:para[child::db:itemizedlist]">
        <div>
            <p>
                <xsl:apply-templates select="*[name() !='itemizedlist'] | text()"/>
            </p>
        </div>
        <xsl:apply-templates select="db:itemizedlist"/>
        <!--<div>
            <p>
                <xsl:apply-templates select="*[name() !='itemizedlist'] | text()"/>
            </p>
        </div>-->
    </xsl:template>
    
    <!-- if a <p> is already opened -->
    <xsl:template match="db:para[name(..) = 'entry' or name(..) = 'footnote']">
        <xsl:apply-templates select="* | text()"/> 
    </xsl:template>
    
    <!-- if there is already an open div -->
    <xsl:template match="db:para[name(..) = 'step' or name(..) = 'answer' or name(..) = 'question']">
        <p>
            <xsl:apply-templates select="* | text()"/>
        </p> 
    </xsl:template>
    
    <!-- if has a variablist as a child, <p> for the content before and after de variablist -->
    <xsl:template match="db:para[name(..) = 'listitem' and child::db:variablelist]">
        <p>
            <xsl:apply-templates select="*[name(.) !='variablelist'] | text()"/>
        </p>
        <xsl:apply-templates select="db:variablelist"/>
        <p>
            <xsl:apply-templates select="*[name(.) !='variablelist'] | text()"/>
        </p>
    </xsl:template>
    
    <!-- para in variable list -->
    <xsl:template match="db:para[name(..) = 'listitem'][not (child::db:variablelist)] [not (name(../..) = 'varlistentry')]"><!-- a revoir -->
        <p>
            <xsl:apply-templates select="* | text()"/>
        </p>
    </xsl:template>
    
    <!-- if parent is a note, a div is already opened -->
    <xsl:template match="db:para[name(..) = 'note']">
        <p>
            <xsl:apply-templates select="* | text()"/>
        </p>
    </xsl:template>
    
    <xsl:template match="db:para[parent::node()/@label = 'footer']">
        <xsl:apply-templates select="db:tag"/>
    </xsl:template>
    
    <!-- if parent is a listitem and grand parent is a varlistentry we already opened a <p> -->
    <xsl:template match="db:para[name(..) = 'listitem'] [name(../..) = 'varlistentry']">
        <xsl:apply-templates select="* | text()"/>
    </xsl:template>
    
    <xsl:template match="db:tag">
        <span>
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <!-- one section for each couple questions / answers -->
    <xsl:template match="db:qandaset">
        <xsl:apply-templates select="db:qandadiv"/>
    </xsl:template>
    
    <!-- each qandadiv is an hdoc div which contains the question and the answer -->
    <xsl:template match="db:qandadiv">
        <section>
            <header>
                <h1><xsl:value-of select="db:title"/></h1>
            </header>
            <xsl:apply-templates select="db:qandaentry"/>
        </section>
    </xsl:template>
    
    <xsl:template match="db:qandaentry">
        <xsl:apply-templates select="db:question | db:answer"/>
    </xsl:template>
    
    <xsl:template match="db:question">
        <div>
            <h6>Question : </h6>
            <xsl:apply-templates select="*"/>
        </div>
    </xsl:template>
    
    <xsl:template match="db:answer">
        <div>
            <h6>Answer : </h6>
            <xsl:apply-templates select="*"/>
        </div>
    </xsl:template>
    
    <!-- in dest, a procedure is a section with a ul li for the steps -->
    <xsl:template match="db:procedure[name(..) = 'article']">
        <section><!-- div ? -->
            <header>
                <h1><xsl:value-of select="db:title"/></h1><!-- h6 ? -->
            </header>
            <div>
                <ul>
                    <xsl:apply-templates select="db:step"/>
                </ul>
            </div>
        </section>
    </xsl:template>
    
    <xsl:template match="db:procedure">
        <div>
            <h6><xsl:value-of select="db:title"/></h6>
            <ul>
                <xsl:apply-templates select="db:step"/>
            </ul>
        </div>
    </xsl:template>
    
    <xsl:template match="db:procedure[name(..) = 'answer']">
        <!-- on perd le titre de la procédure -->
        <ul>
            <xsl:apply-templates select="db:step"/>
        </ul>
    </xsl:template>
    
    <xsl:template match="db:step">
        <li>
            <xsl:apply-templates select="db:para | db:screen"/>
        </li>
    </xsl:template>
    
    <xsl:template match="db:screen[name(..) = 'para']">
        <xsl:apply-templates select="* | text()"/>
    </xsl:template>
    
    <xsl:template match="db:screen">
        <div>
            <p>
                <xsl:apply-templates select="* | text()"/>
            </p>
        </div>
    </xsl:template>
    
	<!-- not in a div if the parent is an answer (because there a div is already opened -->
    <xsl:template match="db:screen[name(..) = 'answer']">
            <p>
                <xsl:apply-templates select="* | text()"/>
            </p>
    </xsl:template>
     
<!-- not in a div if the parent is a step (because there a div is already opened -->	 
    <xsl:template match="db:screen[name(..) = 'step']">
        <p>
            <xsl:apply-templates select="* | text()"/>
        </p>
    </xsl:template>
    
    <xsl:template match="db:replaceable">
        <xsl:value-of select="."/>
    </xsl:template>
    
	<!-- no equivalent to command in hdoc, so it is only emphasized -->
    <xsl:template match="db:command">
        <em><xsl:value-of select="."/></em>
    </xsl:template>
    
    <!-- in dest, example is a an example div -->
    <xsl:template match="db:example">
        <div data-hdoc-type="example">
            <h6><xsl:apply-templates select="db:title"/></h6>
            <p>
                <xsl:apply-templates select="db:programlisting"/>
            </p>
        </div>
    </xsl:template>
    
	<!-- if the example is in a para, we can't open a div so it is just put as a quoted text in the para -->
    <xsl:template match="db:example[name(..) = 'para']">
            <sub>Example: <xsl:apply-templates select="db:title"/></sub>
            <q><xsl:apply-templates select="db:programlisting"/></q>
    </xsl:template>
    
    <!-- in dest, a note is a para with the texte "Note :" at the beginning -->
    <xsl:template match="db:note">
        <div>
            <h6>Note : </h6>
            <xsl:apply-templates select="*"/>
        </div>
    </xsl:template>
    
    <!-- program listing is a div with "Program listing" as title -->
    <xsl:template match="db:programlisting">
        <div>
            <h6>Program listing :</h6>
            <p>
                <q><xsl:value-of select="."/></q>
            </p>
        </div>
    </xsl:template>
    
    <!-- when programlisting is in a para, we can't put another <p> -->
    <xsl:template match="db:programlisting[name(..) != 'section' and name(..) != 'answer']">
        Program listing:
            <q><xsl:value-of select="."/></q> <!-- insérer <! > pour commenter -->
    </xsl:template>
    
	<!-- when programlisting is in an answer, a div is already opened -->
    <xsl:template match="db:programlisting[name(..) = 'answer']">
        <p>
            Program listing:
            <q><xsl:value-of select="."/></q> <!-- insérer <! > pour commenter -->
        </p>
    </xsl:template>
    
	<!-- only image objects are supported for the moment -->
    <xsl:template match="db:mediaobject">
        <xsl:apply-templates select="db:imageobject"/>
    </xsl:template>
    
    <xsl:template match="db:imageobject">
        <img>
            <xsl:attribute name="src">rsc/<xsl:value-of select="db:imagedata/@fileref"/></xsl:attribute>
            <xsl:attribute name="alt"><xsl:value-of select="db:imagedata/@fileref"/></xsl:attribute>
        </img>
    </xsl:template>
    
    <!-- no equivalent for these blocks, so translated to quoted block -->
    <xsl:template match="db:literal">
        <q>
            <xsl:value-of select="."/>
        </q>
    </xsl:template>
    
    <xsl:template match="db:varname">
        <q>
            <xsl:value-of select="."/>
        </q>
    </xsl:template>
    
    <xsl:template match="db:citation">
        <q><xsl:value-of select="."/></q>
    </xsl:template>
    
    <xsl:template match="db:quote">
        <q><xsl:value-of select="."/></q>
    </xsl:template>
    
    <xsl:template match="db:code">
        <q>
            <xsl:value-of select="."/>
        </q>
    </xsl:template>
    
    <xsl:template match="db:computeroutput">
        <q>
            <xsl:apply-templates select="* | text()"/>
        </q>
    </xsl:template>
    
    <!-- no equivalent for these blocks -->
    <xsl:template match="db:uri">
        <xsl:value-of select="."/>
    </xsl:template>
    
    <xsl:template match="db:acronym">
        <xsl:apply-templates select="db:alt | text()"/>
    </xsl:template>
    
    <xsl:template match="db:alt">
        <xsl:value-of select="."/>
    </xsl:template>
    
    <!-- file name remplacé par rien -->
    <xsl:template match="db:filename">
        <xsl:value-of select="."/>
    </xsl:template>
    
    <xsl:template match="db:footnote">
        <xsl:apply-templates select="db:para"/>
    </xsl:template>
    
    <xsl:template match="db:phrase">
        <xsl:value-of select="."/>
    </xsl:template>
    
    <!-- link between sections doesn't exist in hdoc -->
    <xsl:template match="db:xref">
        section "<xsl:value-of select="@linkend"/>" 
    </xsl:template>
    
    <xsl:template match="db:citetitle">
        <sub><xsl:value-of select="."/></sub>
    </xsl:template>
    
    <xsl:template match="db:menuchoice">
        <xsl:apply-templates select="db:guimenu | db:guimenuitem"/>
    </xsl:template>
    
    <!-- menu and labels are replaced by emphasized -->
    <xsl:template match="db:guimenu | db:guimenuitem | db:guilabel | db:guibutton">
        <em><xsl:value-of select="."/></em>
    </xsl:template>
    
    <!-- in dest, application is a link -->
    <xsl:template match="db:application">
        <xsl:variable name="lien" select="./@xl:href"/>
        <a href="{$lien}">
            <xsl:if test=". != ''">
                <xsl:apply-templates select="text() | db:citetitle"/>
            </xsl:if>
            <xsl:if test=". = ''">
                <xsl:value-of select="./@xl:href"/>
            </xsl:if>
        </a>
    </xsl:template>
    
    <xsl:template match="db:emphasis">
                <em><xsl:value-of select="."/></em>            
    </xsl:template>
    
    <!-- if the link has an associated text, put the text, if not put the link address as text -->
    <xsl:template match="db:link">
        <xsl:variable name="lien" select="./@xl:href"/>
        <a href="{$lien}">
            <xsl:if test=". != ''">
                <xsl:apply-templates select="text() | db:citetitle"/>
            </xsl:if>
            <xsl:if test=". = ''">
                <xsl:value-of select="./@xl:href"/>
            </xsl:if>
        </a>
    </xsl:template>
    
    <!-- itemizedlists are parsed to ul li -->
    <xsl:template match="db:itemizedlist">
        <div>
            <ul>
                <xsl:apply-templates select="db:listitem"/>
            </ul>
        </div>
    </xsl:template>
    
    <!-- when we are already in a div -->
    <xsl:template match="db:itemizedlist[name(..) = 'answer']">
        <ul>
            <xsl:apply-templates select="db:listitem"/>
        </ul>
    </xsl:template>
    
    <xsl:template match="db:listitem[name(..) = 'varlistentry']">
        <xsl:apply-templates select="db:para"/>
    </xsl:template>
    
    <xsl:template match="db:listitem">
        <li>
            <xsl:apply-templates select="db:para"/>
        </li>
    </xsl:template>
    
    <xsl:template match="db:table">
        <div>
            <h6><xsl:value-of select="db:title"/></h6>
            <table>
                <caption>
                    <xsl:value-of select="db:title"/>
                </caption>
                <xsl:apply-templates select="db:tgroup"/>
            </table>
        </div>
    </xsl:template>
    
    <xsl:template match="db:tgroup">
        <xsl:apply-templates select="db:tbody | db:thead"/>
    </xsl:template>
    
    <xsl:template match="db:tbody | db:thead">
        <xsl:apply-templates select="db:row"/>
    </xsl:template>
    
    <xsl:template match="db:row">
        <tr>
            <xsl:apply-templates select="db:entry"/>
        </tr>
    </xsl:template>
    
    <xsl:template match="db:entry">
        <td>
            <p>
                <xsl:apply-templates select="* | text()"/>
            </p>
        </td>
    </xsl:template>
    

    <!-- variablelists, as itemizedlists, are parsed to ul li -->    
    <xsl:template match="db:variablelist">
        <div>
            <ul><xsl:apply-templates select="db:varlistentry"/></ul>
        </div>
    </xsl:template>
    
    <xsl:template match="db:variablelist[name(..) = 'para']">
            <ul><xsl:apply-templates select="db:varlistentry"/></ul>
    </xsl:template>
    
    <xsl:template match="db:varlistentry">
        <li>
            <p>
                <xsl:value-of select="db:term"/> :
                <xsl:apply-templates select="db:listitem"/>
            </p>
        </li>
    </xsl:template>
    
    <xsl:template match="db:figure">
        <div>
            <h6>Figure : <xsl:value-of select="db:title"/></h6>
            <xsl:apply-templates select="db:mediaobject"/>
        </div>
    </xsl:template>
    
</xsl:stylesheet>